
from optparse import OptionParser

parser = OptionParser()
parser.add_option("-H", "--headless", action="store_true", dest="headless", help="Use headless mode of the browser", default=False)
parser.add_option("-u", "--url-file", dest="url_file", help="A markdown file containing URLs to download", metavar="URL_MARKDOWN_FILE")
parser.add_option("-d", "--delete-element-file", dest="delete_element_file", help="A JSON file contains CSS selectors of which elements to delete", metavar="DELETE_ELEMENT_JSON_FILE")
parser.add_option("-j", "--js-file", dest="js_file", help="A JS file to execute", metavar="JS_FILE")
parser.add_option("-c", "--cover", dest="cover", help="Cover page, local image file or an URL of an image", metavar="COVER_IMAGE", default=None)
parser.add_option("-o", "--output", dest="output_file", help="The output file", metavar="OUTPUT_FILE")
parser.add_option("-w", "--wait-seconds", dest="wait_seconds", help="Wait for the page to load", metavar="WAIT_SECONDS", default=30)
parser.add_option("-I", "--ignore-wait-error", dest="ignore_wait_error", help="Ignore wait error", action="store_true", default=False)

(options, args) = parser.parse_args()

if not options.url_file:
    print("URL_MARKDOWN_FILE is required\n")
    parser.print_help()
    exit(1)


if not options.output_file:
    print("OUTPUT_FILE is required\n")
    parser.print_help()
    exit(1)


if not options.delete_element_file:
    print("DELETE_ELEMENT_JSON_FILE is required\n")
    parser.print_help()
    exit(1)


from dataclasses import dataclass
from mistletoe import Document
from mistletoe.token import Token
from mistletoe.span_token import Link

@dataclass
class URL:
    url: str
    title: str

def get_links(token: Token) -> list[Link]:
    if isinstance(token, Link):
        return [token]
    if "children" in vars(token):
        links = []
        for child in token.children:
            links.extend(get_links(child))
        return links
    return []

def parse(filename: str) -> list[URL]:
    with open(filename, 'r') as f:
        md = Document(f)
        return [
            URL(link.target, link.children[0].content)
            for link in get_links(md)
        ]


import os
import tempfile
from PyPDF2 import PdfWriter
import requests
from PIL import Image
from webook import web
from webook.cli import options
from webook.urls import parse

def make_cover(cover: str, tempdir: str) -> str:
    """
    Conver an image to a pdf page

    Parameters:
    -----------
    cover: str
        The path or URL to the image file
    tempdir: str
       The path to the temporary directory

    Returns:
    --------
    str
        The path to the PDF file
    """

    if os.path.exists(cover):
        img = Image.open(cover)
    else:
        img = Image.open(requests.get(cover, stream=True).raw)
    blank_image = Image.new('RGB', (595, 842), 'white')
    img = img.resize((595, int(595 * img.height / img.width)))
    blank_image.paste(img, ((595 - img.width) // 2, (842 - img.height) // 2))
    pdf = f"{tempdir}/Cover.pdf"
    blank_image.save(pdf, "PDF", resolution=72.0, save_all=True)
    return pdf

def merge(pdfs: list[str], output_file: str):
    merger = PdfWriter()

    for filename in pdfs:
        input_pdf = open(filename, 'rb')
        outline_name = os.path.basename(filename)
        outline_name = os.path.splitext(outline_name)[0]
        merger.append(input_pdf, outline_item=outline_name)

    output_pdf = open(output_file, 'wb')
    merger.write(output_pdf)
    merger.close()
    output_pdf.close()


def main():
    with tempfile.TemporaryDirectory() as tempdir:
        pdfs = []
        if options.cover:
            pdfs.append(make_cover(options.cover, tempdir))
        pdfs.extend(web.save_as_pdf(parse(options.url_file), tempdir, options))
        merge(pdfs, options.output_file)


if __name__ == "__main__":
    main()



import json


def parse(filename: str) -> dict[str, list[str]]:
    with open(filename, "r") as f:
        file_json = json.load(f)
        return {
            k.replace("web:", "", 1): [s.get('selector', None) for s in json.loads(v)]
            for (k, v)
            in file_json.items()
            if k.startswith("web:")
        }

def for_url(url: str, filename: str) -> list[str]:
    all_selectors = parse(filename)
    return next((v for (k, v) in all_selectors.items() if k in url), [])


from optparse import Values
from webook import selectors
from webook.urls import URL
from playwright.sync_api import sync_playwright, TimeoutError

def save_as_pdf(links: list[URL], tempdir: str, options: Values) -> list[str]:
    pdfs = []
    with sync_playwright() as p:
        browser = p.chromium.launch(headless=options.headless)
        page = browser.new_page()
        for link in links:
            page_selectors = selectors.for_url(link.url, options.delete_element_file)
            try:
                page.goto(link.url, timeout=options.wait_seconds * 1000)
            except TimeoutError:
                if not options.ignore_wait_error:
                    raise
            # page.wait_for_event('domcontentloaded')
            # remove elemtents
            for selector in page_selectors:
                if selector:
                    for elem in page.query_selector_all(selector):
                        elem.evaluate('e => e.remove()')

            if options.js_file:
                with open(options.js_file, 'r') as f:
                    js = f.read()
                    page.evaluate(js)

            # save as PDF
            pdf = f'{tempdir}/{link.title}.pdf'
            page.pdf(path=pdf)
            pdfs.append(pdf)
    return pdfs